/* Fetch list of newsgroups */
call RxFuncAdd 'SysLoadFuncs','RexxUtil','SysLoadFuncs'
call SysLoadFuncs
call RxFuncAdd 'SockLoadFuncs','RxSock','SockLoadFuncs'
signal on SYNTAX
nosock=1
call SockLoadFuncs 'q'
nosock=0
signal off SYNTAX

rxiversion=0.09
parse version rxversion
if word(rxversion,1)\='OBJREXX' then do
    say ''
    say 'NGFETCH.CMD requires Object REXX to function.'
    say ''
    say 'You are running: '||rxversion
    say ''
    say 'README.TXT contains more information.'
    exit 4
end

signal on HALT

parse arg host switch

if host=''|host='?'|host='/?'|host='-?' then do
    say ''
    say 'News group list fetcher v'||rxiversion
    say ''
    say 'Usage: ngfetch.cmd <host> [/raw] [/q]'
    say ''
    say 'host - hostname of news server'
    say ''
    say '/raw - write all received info without parsing'
    say ''
    say '/q - operate in quiet mode'
    exit
end

if switch~translate~pos('/Q')>0 then quietmode=1
else quietmode=0
if switch~translate~pos('/RAW')>0 then writeraw=1

crlf='0d0a'x
blocksize=10240
call SysCurState 'off'
con=.stream~new('stdout')
listend='2e0d0a'x
exitcode=0

outfile=.stream~new('.newsrc')
call SysFileDelete outfile~qualify

port=119

sendstring='LIST'||crlf

csock=.socket~new

call SysCls
con~charout(crlf~copies(5))
con~close
say 'Connecting to host '||host||' on port '||port||'...'
check=csock~connect(host||':'||port)
if check='HALT' then signal HALT
if check=-1 then do
    say 'Connection to host '||host||' on port '||port||' failed!'
    csock~shutdown('both')
    csock~close
    exit 1
end
say ''

say 'Requesting group list...'
check=csock~send(sendstring)
if check='HALT' then signal HALT
if check=-1 then do
    say 'Failed to send group request!'
    csock~shutdown('both')
    csock~close
    exit 1
end
say ''

say 'Receiving group list... '
say ''
if quietmode\=1 then do
    parse value SysCurPos() with row col
    grow=row+2
    call SysCurPos row,0
    con~charout('Bytes received: ')
    con~close
    call SysCurPos row,32
    con~charout('Bytes/second: ')
    con~charout(crlf||crlf)
    con~close
    if writeraw\=1 then do
        call SysCurPos grow,5
        con~charout('Groups received: ')
        con~close
    end     
calc=0
next=0
ngr=100
gstep=100
freq=10240
freq2=1024
end
total=0
data=''
groups=0
call time 'r'
check=csock~receive(blocksize)
if check='HALT' then signal HALT
total=total+check[1]~length
if check[1]~word(1)='400' then do
    if quietmode\=1 then call SysCurPos row,70
    say ''
    say ''
    say 'News server disconnected (probably exceeded per host connection limit).'
    say ''
    csock~shutdown('both')
    csock~close
    exit 2
end
else if writeraw=1 then outfile~charout(check[1])

do until check[1]~right(3)=listend
    check=csock~receive(blocksize)
    if check='HALT' then signal HALT
    rvar=check[2]
    total=total+check[1]~length
    data=data||check[1]
    if quietmode\=1 then do
        if total>=next | data~right(3)=listend then do
            ct=time('e')
            next=total+freq2
            call SysCurPos row,16
            con~charout(codel(total)~left(12))
            con~close
            if total>50000 then freq2=rate%2
        end
        if total>=calc | data~right(3)=listend then do
            rate=total%ct
            calc=total+freq
            if total>50000 then freq=rate%2
            call SysCurPos row,46
            con~charout(codel(rate)~left(12))
            con~close
        end
    end
    if writeraw=1 then do
        outfile~charout(data)
        data=''
    end
    else do
        if data~word(1)='215' then do
            parse var data '215' . (crlf) data
        end
        do while data~pos(crlf)>0
            parse var data group ' ' (crlf) data
            outfile~lineout(group)
            groups=groups+1
            if data~left(3)=listend then leave
        end        
        if quietmode\=1 then do
            if groups>=ngr | data~left(3)=listend then do
                ngr=groups+gstep
                gstep=rate%80
                call SysCurPos grow,23
                con~charout(codel(groups)~left(8))
                con~close
            end
        end
    end
end 
ct=time('e')
rate=total%ct
outfile~close 
if quietmode\=1 then do
    say ''
    say ''
end
say codel(total)||' bytes transferred in '||ct~format(,3)||' seconds ('||,
    codel(rate)||' bytes/sec.)'
say ''
if writeraw\=1 then do
    say codel(groups)||' groups received.'
    say ''
end
exit

HALT:

if datatype(grow,'NUM')=1 then call SysCurPos grow,70
say ''
say ''
say 'Aborting!'
if csock~objectname='a socket' then do
    csock~shutdown('both')
    csock~close
end
exit 3

SYNTAX:

if nosock=1 then do
    say ''
    say 'Failed to load REXX Socket functions.'
    say ''
    say 'Program cannot continue.'
end
exit 4

/* Socket class - work in progress **********************************/
/*                                                                  */
::class 'socket'

::method init
expose portnum
portnum=SockSocket('AF_INET','SOCK_STREAM',0)
return

::method sock
expose portnum
return portnum

::method connect
use arg servname
signal on HALT
parse var servname hostname ':' host.!port
parse var hostname o1 '.' o2 '.' o3 '.' o4
if o1~datatype='NUM' & o2~datatype='NUM' & o3~datatype='NUM' &,
   o4~datatype='NUM' then do
    if o1~datatype('w')=1 & o2~datatype('w')=1 & o3~datatype('w')=1 &,
       o4~datatype('w')=1 then do
        if (o1>=0 & o1<=255) & (o2>=0 & o2<=255) & (o3>=0 & o3<=255) &,
           (o4>=0 & o4<=255) then do
            stype='IP'
            host.!addr=hostname
        end
    end
end
else stype='NAME'            
if stype='NAME' then call SockGetHostByName hostname,'host.!'
host.!family='AF_INET'
rc=SockConnect(self~sock,'host.!')
return rc
HALT:
return 'HALT'

::method send
use arg sendstring
signal on HALT
rc=SockSend(self~sock,sendstring)
return rc
HALT:
return 'HALT'

::method receive
use arg length
signal on HALT
if length='LENGTH' then length=1024
check=SockRecv(self~sock,'sockdata',length)
return .array~new~~put(sockdata,1)~~put(check,2)
HALT:
return 'HALT'

::method shutdown
use arg how
select
    when how~translate='FROM' then how=0
    when how~translate='TO' then how=1
    otherwise how=2
end
call SockShutDown self~sock,how
return

::method close
call SockClose self~sock
return
/*                                                                  */
/********************************************************************/

/* Comma deliminator routine ****************************************/
/*                                                                  */
::routine codel
use arg RNum
if RNum~length<3 then return RNum
RNum=RNum~reverse
FNum=''
do while RNum~length>3
    parse var RNum TriDig +3 RNum
    FNum=FNum||TriDig||','
end
FNum=FNum||RNum
return FNum~reverse
/*                                                                  */
/********************************************************************/
